% create video from image files stored in the selected directory
% this script requires MATLAB 2016b or later
% - image file order (by name) in the selected folder must correspond to
%   the order in the video

% input directory containing image files
imageDir = fullfile(filesep, 'misc', '7', 'output', 'grosse', ...
                    'CREG_project', 'CREG025E-TOP-DYE06_fg001', ...
                    'MATLAB_output', 'figures', 'maps_intDepths', ...
                    'monthly', 'dep0-40', 'dye01_age', 'BGyre');

% image file type (ending)
imageType = 'png';

% image file starting string
imageStart = 'CREG';

% video output directory
videoDir = imageDir;

% video file name (file ending is determined automatically based on video file codec)
videoFile = 'Video_dye01_age_Pacific_dye_0-40m_19630101-19651231';

% choose video properties
% video file encoding
% =========================================================================
% - available options are (provided all codecs are installed):
%   'Archival'         ==> Motion JPEG 2000 file with lossless compression
%   'Motion JPEG AVI'  ==> AVI file using Motion JPEG encoding
%   'Motion JPEG 2000' ==> Motion JPEG 2000 file
%   'MPEG-4'           ==> MPEG-4 file with H.264 encoding (systems with Windows 7 or later, or Mac OS X 10.7 and later)
%   'Uncompressed AVI' ==> Uncompressed AVI file with RGB24 video
%   'Indexed AVI'      ==> Uncompressed AVI file with indexed video
%   'Grayscale AVI'    ==> Uncompressed AVI file with grayscale video
% =========================================================================
videoCodec = 'Motion JPEG AVI';

% video frame rate (frames per second, positive number)
% value <=0 implies default value (10)
videoFR = 3;

% video quality (0 to 100)
% value <=0 implies default quality (75)
videoQU = 75;

% use lossless compression? (only available for 'Archival' encoding)
videoLLC = false;

% compression ratio (1 to 10: no compression to 10-fold compression)
% value <=0 implies default compression (10)
videoCR = 0;


%% START PROGRAM
% retrieve file list
dirOut = dir(fullfile(imageDir, sprintf('%s*.%s', imageStart, imageType)));
nFiles = length(dirOut);
if  nFiles<=1
    fprintf('Folder contains at most one file matching the file name start (''%s'') and file type (''%s'').\nNo video created.', ...
            imageStart, imageType)
    return
end

% get individual file names and figure sizes
files = cell(1,nFiles);
for i = 1:nFiles
    files{i} = fullfile(dirOut(i).folder, dirOut(i).name);
end

%% INITIALIZE VIDEO
% default values for frame rate, quality, and compression rate
FRdef = 10;
QUdef = 75;
CRdef = 10;

% check selected video options
% get list of installed codeca and check if selected encoding is available
profAvail = VideoWriter.getProfiles();
isAvail = sum(strcmp({profAvail.Name}, videoCodec));
if  isAvail==0
    profList = '';
    for i = 1:length(profAvail)
        profList = sprintf('%s > %25s: %s\n', profAvail(i).Name, profAvail(i).Description);
    end
    error(['The selected video encoding (''%s'') is currently not installed on your computer.\n', ...
           'Either install the selected codec or select a different encoding.\n', ...
           'Available codecs are:\n%s'], profList);
end

% check codec settings
profStr = replace(videoCodec, {' ', '-', 'JPEG'}, {'', '', 'Jpeg'});
eval(sprintf('profInfo = audiovideo.writer.ProfileInfo(audiovideo.writer.profile.%s);', profStr));
videoType = profInfo.FileExtensions{1};
switch videoCodec
    case {'Archival', 'Motion JPEG 2000'}
        videoQU = QUdef;
    case {'Motion JPEG AVI', 'Uncompressed AVI', 'Indexed AVI', 'Grayscale AVI'}
        videoLLC = false;
        if ~strcmp(videoCodec, 'Motion JPEG AVI'), videoQU = QUdef; end
    case 'MPEG-4'
        videoLLC = false;
end

videoName = sprintf('%s%s', videoFile, videoType);
if  videoLLC
    videoLLCstr = 'on';
else
    videoLLCstr = 'off';
end

% set properties to default values if applicable
if videoFR<=0, videoFR = FRdef; end
if videoQU<=0, videoQU = QUdef; end
if videoCR<=0, videoCR = CRdef; end

% report video properties
props = fieldnames(profInfo);
fprintf('Create video with the following properties:\n')
fprintf(' > file directory      : %s\n', videoDir);
fprintf(' > file name           : %s\n', videoName);
fprintf(' > video encoding      : %s\n', videoCodec);
fprintf(' > number of frames    : %i\n', nFiles);
if  sum(strcmp(props,'FrameRate'))==1
    fprintf(' > video frame rate    : %i\n', videoFR);
end
if  sum(strcmp(props,'Quality'))==1
    fprintf(' > video quality       : %i\n', videoQU);
end
if  sum(strcmp(props,'LosslessCompression'))==1
    fprintf(' > lossless compression: %s\n', videoLLCstr);
end
if  sum(strcmp(props,'CompressionRatio'))==1
    fprintf(' > compression rate    : %i\n', videoCR);
end


% create video object and set properties
if ~exist(videoDir, 'dir'), mkdir(videoDir); end
vObj = VideoWriter(fullfile(videoDir, videoName), videoCodec);
if  sum(strcmp(props,'FrameRate'))==1
    vObj.FrameRate = videoFR;
end
if  sum(strcmp(props,'Quality'))==1
    vObj.Quality = videoQU;
end
if  sum(strcmp(props,'LosslessCompression'))==1
    vObj.LosslessCompression = videoLLC;
end
if  sum(strcmp(props,'CompressionRatio'))==1
    vObj.CompressionRatio = videoCR;
end
fprintf('Video properties set successfully.\n\n')

% check maximum dimensions of images to be read
fprintf('Get maximum image dimensions of all images ...\n')
imageWidth = 0;
imageHeight = 0;
for i = 1:nFiles
    fInf = imfinfo(files{i});
    imageWidth = max([imageWidth, fInf.Width]);
    imageHeight = max([imageHeight, fInf.Height]);
end
% ensure that image dimensions are even numbers
% (required for potential later conversion from any format to mp4)
if mod(imageWidth,2)~=0 , imageWidth  = imageWidth  + 1; end
if mod(imageHeight,2)~=0, imageHeight = imageHeight + 1; end
fprintf('Maximum image dimensions are: %i x %i (width x height, in pixels).\n\n', ...
        imageWidth, imageHeight);

%% open video video and write images
fprintf('Write video ...\n');
nDigits = floor(log10(nFiles)+1);
frmt = ['Write frame #%0', num2str(nDigits), 'i: %s\n'];
open(vObj);
for i = 1:nFiles
    fprintf(frmt, i, files{i});
    imageData = NaN(imageHeight, imageWidth, 3);
    imageData(:) = 255;
    imageIn = imread(files{i}, imageType);
    eval(sprintf('imageData = %s(imageData);', class(imageIn)));
    if  size(imageIn,1)<imageHeight
        iStart = imageHeight - size(imageIn,1) + 1;
    else
        iStart = 1;
    end
    jEnd = min([imageWidth, size(imageIn,2)]);
    imageData(iStart:end, 1:jEnd, :) = imageIn;
    writeVideo(vObj, imageData);
end
close(vObj);
fprintf('\nVideo creation completed successfully.\n');